CellDataBaseWrapper = CellDataBaseWrapper or blt_class()

function CellDataBaseWrapper:init(name, debug_enabled)	
	FileUtils:LoadFile(CellDataBlockManager.CELLDATA_SCRIPT_PATH ..name..".lua");
	self._name = name	
	self._debug_enabled = debug_enabled
	self._is_setup = false
	self._wrapper = nil
	self._fileIsDirty = false
	self._m_uniqueInt = 0;
	self._block_class = rawget(_G, self._name)	
	self._descriptors = {}	
	if not self._block_class then
		error("[CellDataBaseWrapper:post_setup] class for block type " .. self._name .. " doesnt exist!!!")
	end	
	self._fileNameNoExt = "CellData." .. self._name
	self._fileName = self._fileNameNoExt .. ".json"
	self._fileBytesName = self._fileNameNoExt .. ".bytes"
	self._fileHeader = self._fileNameNoExt .. "_header.lua"
	self._headerPath = CellDataBlockManager.DATABLOCKS_HEADER_PATH .. self._fileHeader 
	self._filePathRes = CellDataBlockManager.DATABLOCKS_RESOURCE_PATH .. self._fileName;
	self._filePath = CellDataBlockManager.DATABLOCKS_PATH .. self._fileName;
	self._filePathBin = CellDataBlockManager.DATABLOCKS_PATH .. self._fileBytesName;
	self._callbacks_on_post_setup = {}
	self:setup();
end 

function CellDataBaseWrapper:name()	
	return self._name
end

function CellDataBaseWrapper:wrapper()	
	return self._wrapper
end

function CellDataBaseWrapper:Blocks()	
	return self._wrapper.Blocks
end

function CellDataBaseWrapper:wrapper_name()	
	return self._name:gsub("DataBlock","");
end


function CellDataBaseWrapper:add_callback_on_post_setup(clbk)	
	if self._is_setup then
		clbk()
	else
		table.insert(self._callbacks_on_post_setup, clbk)
	end
end

function CellDataBaseWrapper:clean_block(block)
end

function CellDataBaseWrapper:Serialize(indent)
	local writer = JsonWriter:new(self._wrapper, indent)
	writer:Write("Headers", "array")
	writer:Write("Blocks", "array")
	writer:WriteFinal("LastPersistentID", "number")	
	return writer:Results()
end



function CellDataBaseWrapper:get_file_contents()
	if not CellDataBlockManager.IS_RESOURCE_FILE then		
		if CellDataBlockManager.LOAD_BINARY then
			if FileUtils:PathExists(self._filePathBin) then
				return BinaryEncoder:Decode(CellDataBlockManager.m_binaryEncoder, FileUtils:ReadAllBytes(self._filePathBin))	
			end
		elseif FileUtils:PathExists(self._filePath) then
			return FileUtils:ReadAllText(self._filePath)	
		end		
	elseif FileUtils:PathExists(self._filePathRes) then
		return FileUtils:ReadAllText(self._filePathRes)	
	end		
	return nil
end

function CellDataBaseWrapper:setup()
	self:base_setup()
end

function CellDataBaseWrapper:base_setup()
	self:LoadFromDisk()
	self:add_descriptor("persistentID", DataDescriptor:new({
		dataLayoutType = DataDescriptor.DataLayoutType.Hidden
	}))	
	
	self._is_setup = true
end

function CellDataBaseWrapper:LoadFromDisk()
	local fileContents = self:get_file_contents()	
	if fileContents then
		local file_data = JsonUtils:safe_decode(fileContents)
	
		if not file_data then
			self._fileIsDirty = true
			error("[CellDataBaseWrapper:setup_blocks] " ..self._name .. " failed to parse!  Report this crash to the developer....  In the mean time uninstall ProjectCell...")		
			return
		end
		
		if type(file_data) == "number" then
			self._fileIsDirty = true
			error("[CellDataBaseWrapper:setup_blocks] " ..self._name .. " has returned a number value! This is suppose to be a string value. Report this crash to the developer....  In the mean time uninstall ProjectCell...")
			return
		end
		
		self._blocks_by_IDS = {}		
		self._blocks_by_names = {}		
		self._blockIDs_by_names = {}
		
		self._wrapper = file_data;		
		local builder = StringBuilder:new()
		if (not self._wrapper.LastPersistentID)  then
			self._wrapper.LastPersistentID = 0;	
			self._fileIsDirty = true		
			builder:AppendLine("\t\t<<Didn't have a LastPersistentID. Manually asssigning one>>")
		end
		
		
		
		if (not self._wrapper.Headers)  then
			self._wrapper.Headers = {};		
			self._fileIsDirty = true
			builder:AppendLine("\t\t<< didn't have a Headers. Manually creating one>>")
		end
		
		--SerializableObject.apply_class_to_array(self._wrapper, "Headers", "CellDataHeader")	
		
		for i, block in pairs(self._wrapper.Blocks) do
			

			if block.internalEnabled == nil or type(block.internalEnabled) ~= "boolean" then
				block.internalEnabled = false
				self._fileIsDirty = true
				builder:AppendLine("\t\t<<A block has the field internalEnabled that is NOT a Boolean!!>>")
			end
			
			if not block.persistentID then
				block.persistentID = 0;
				self._fileIsDirty = true
				builder:AppendLine("\t\t<<A block didn't have a persistentID field in Wrapper !!>>")
				self:CheckPersistentID(block, false);
			end
			
			if block.persistentID == 0 then
				self._fileIsDirty = true
				builder:AppendLine("\t\t<<A block has a persistentID of 0, reassigning!!>>")
				self:CheckPersistentID(block, false);
			end
			
			if not block.name then
				block.name = "UnnamedBlock";
				self._fileIsDirty = true
				builder:AppendLine("\t\t<<A block didn't have a name field in Wrapper!!>>")
			end

			if self._blocks_by_IDS[block.persistentID] then			
				builder:AppendLine("\t\t<<Block " ..block.name.." "..tostring(block.persistentID).."   has a duplicate ID, reassigning. >>")
				self:CheckPersistentID(block, true);	
			end

			if self._blocks_by_names[block.name] then
				builder:AppendLine("\t\t<<Block " ..block.name.." "..tostring(block.persistentID).."   has a duplicate NAME, renaming! >>")
				self:EnsureUniqueName(block, self._blocks_by_names, "", true)			
				builder:AppendLine("\t\t<<Block has been rennamed  to " ..block.name..">>")
			end
			
			
			
			local classed_block = self._block_class:load(block)
			
			
			
			--self:clean_block(classed_block)
			
			self._wrapper.Blocks[i] = classed_block
			
			self._blocks_by_IDS[classed_block.persistentID] = classed_block			
			self._blocks_by_names[classed_block.name] = classed_block			
			self._blockIDs_by_names[classed_block.name] = classed_block.persistentID			
		end
		
		self:CheckLastPersistentID();	
		
		
		self:debug_print(" <<<<<<<<<<<Setup complete!>>>>>>>");	
		
		for i, clbk in pairs(self._callbacks_on_post_setup) do
			clbk()
			table.remove(self._callbacks_on_post_setup,  i)
		end
		
		
		
		
		if self._fileIsDirty then
			self:SaveToDisk()
			self._fileIsDirty = false
			self:debug_print(" <<<<<<<NOTE: This file is dirty and had to be fixed during runtime, it should be cleaned out and saved....>>>>>>");	
			self:debug_print( builder:Results())
		else
			self:debug_print("CLEAN!!!!");	
		end
		
		
		--[[if (not FileUtils:DirectoryExists(CellDataBlockManager.DATABLOCKS_HEADER_PATH)) then
			error("[CellDataBaseWrapper:init] ["..self._name.."] ERROR: " ..CellDataBlockManager.DATABLOCKS_HEADER_PATH.." doesn't exist! Did we delete this folder?")
		end
		
		
		if not FileUtils:LoadFile(self._headerPath) then
			log("[CellDataBaseWrapper:init] ["..self._name.."] ERROR: " ..self._headerPath.." doesn't exist! We need this header file in order to utilize the data blocks more efficiently.")
		end]]
		return		
	end
	
	self:debug_print("No Wrapper for "..self._name.." starting off fresh")
	self._blocks_by_IDS = {}		
	self._blocks_by_names = {}		
	self._blockIDs_by_names = {}
	self._wrapper = {}
	self._wrapper.Headers = {}
	self._wrapper.Blocks = {}
	self._wrapper.LastPersistentID = 0	
end

function CellDataBaseWrapper:add_descriptor(key, descriptor, override_existing)
	local descriptors = string.split(key, ",")
	for _, desc in pairs(descriptors) do
		if override_existing or not self._descriptors[key] then
			self._descriptors[key] = desc
		end	
	end	
end

function CellDataBaseWrapper:Toggle(force)
	if MenuUI and (force or not self._menu) then
		self._menu = MenuUI:new({
			name = self:wrapper_name()	.. "_editor",
			create_items = callback(self, self, "CreateItems"),
		})
	end
	if self._menu then
		self._menu:Toggle()
	end	
end

function CellDataBaseWrapper:inspect_block(inspect_block,menu)
	local holder = menu:PopupMenu({
		name = inspect_block.name .. "_inspector", 
		auto_height = true, 
		background_color = Color(0.8, 0.2, 0.2, 0.2),
		localized = false
	})

	
	holder:TextBox({
		name = "Block_Name",
		text = "Name",
		value = tostring(inspect_block.name),
		on_callback = function(item)
			inspect_block.name = tostring(item:Value())
		end
	})
		
	holder:Button({
		name = "project_cell_block_close",
		text = "Close",
		on_callback = function(item)
			menu:SetEnabled(false)
		end
	})
end

function CellDataBaseWrapper:CreateItems(menu)
	local holder = menu:Menu({name = self:wrapper_name() .. "_block_Editor", auto_height = true, localized = false})	
	
	for _,block in pairs(self._wrapper.Blocks) do
		holder:Button({
			name = block.name,
			text = block.name,
			on_callback = callback(self,self,"inspect_block", block)
		})
	end
	
	holder:Text("block_divider1")
	holder:Text("block_divider2")
	holder:Text("block_divider3")
	holder:Text("block_divider4")
	
	
	
	holder:Button({
		name = "project_cell_save_blocks",
		text = "SaveToDisk",
		on_callback = function(item)
			self:SaveToDisk()
		end
	})
	
	holder:Button({
		name = "project_cell_save_blocks",
		text = "Add New Block",
		on_callback = function(item)
			self:add_new_block()		
			self:Toggle()
		end
	})

	holder:Button({
		name = "project_cell_close",
		text = "Close",
		on_callback = function(item)
			self._menu:Destroy()
		end
	})
end

function CellDataBaseWrapper:LogMessage(function_name, message)
	if self._debug_enabled then
		log("["..self._name.."."..function_name.."] ["..self._name.."]  "..message);
	end	
end

function CellDataBaseWrapper:debug_print(message)
	if self._debug_enabled then
		log(self._name .. " - " .. message)
	end	
end


function CellDataBaseWrapper:CheckLastPersistentID()
	self._wrapper.LastPersistentID = 0
	for _, block in pairs(self._wrapper.Blocks) do
		self._wrapper.LastPersistentID = math.max(self._wrapper.LastPersistentID, block.persistentID);
	end
	
	self:debug_print(" updated LastPersistentID : "..  self._wrapper.LastPersistentID);
end

function CellDataBaseWrapper:CheckPersistentID(block, force)
	if force or block.persistentID <= 0 then
		self:CheckLastPersistentID();		
		self._wrapper.LastPersistentID = self._wrapper.LastPersistentID + 1;		
		block.persistentID = self._wrapper.LastPersistentID;		
		--self:LogMessage("CheckPersistentID", "Setting persistentID to: " .. block.persistentID);
		self._fileIsDirty = true
	end	
end

function CellDataBaseWrapper:EnsureUniqueName(block, existingNames, originalName, first)

	if first then
		self._m_uniqueInt = 0;
		originalName = block.name;
	end
	
	local nameExists = existingNames[block.name]
	
	if nameExists  then
		self._m_uniqueInt = self._m_uniqueInt + 1;		
		block.name = originalName .. "_" ..self._m_uniqueInt;
		self:EnsureUniqueName(block, existingNames, originalName, false);
	end
	
end

function CellDataBaseWrapper:print_results()
	local results = ""
	for _,block in pairs(self._wrapper.Blocks) do
		results = results.."\t\tblock.name: "..block.name.."\n"
		results = results.."\t\tblock.persistentID: "..block.persistentID.."\n"
		results = results.."\t\tblock.internalEnabled : "..tostring(block.internalEnabled).."\n\n"			
	end
	
	self:debug_print(" results:\n"..results);
	
	if self._fileIsDirty then
		self:debug_print("print_results", " this file is dirty, must stabilize it before releasing..");
	end
end

function CellDataBaseWrapper:editor()
end


function CellDataBaseWrapper:post_setup()
end

function CellDataBaseWrapper:PostLoadFromDisk()
	if self._fileIsDirty then
		self:SaveToDisk()
		self._fileIsDirty = true
	end
end


function CellDataBaseWrapper:generateHeaderFile()
	self:debug_print("Generating Header File....")	
	local name = self._name:gsub("DataBlock","")
	local stringBuilder = StringBuilder:new()
	
	stringBuilder:AppendLine("CD = CD or {}")
	stringBuilder:AppendLine("CD."..name.. " = {}")
	for index, block in pairs(self._wrapper.Blocks) do
		if block.name then
			local text = block.name			
			text = text:gsub('%W','_')			
			text = text:gsub("-","_")
			text = text:gsub(";","_")
			text = text:gsub(",","_")		
			stringBuilder:AppendLine("CD."..name.."."..text .. " = " .. block.persistentID)		
		end		
	end	
	FileUtils:WriteAllText(self._headerPath, stringBuilder:Results())
end

function CellDataBaseWrapper:SaveToDisk()
	self:DoSaveToDisk(false,false)
end

function CellDataBaseWrapper:SaveToDiskAndGenerateHeader()
	self:DoSaveToDisk(true,false)
end

function CellDataBaseWrapper:SaveToDiskBinary()
	self:DoSaveToDisk(false,true)
end

function CellDataBaseWrapper:DoSaveToDisk(generateHeaderFile, saveBinary)
	saveBinary = saveBinary and true or false
	if FileUtils:DirectoryExists(CellDataBlockManager.DATABLOCKS_PATH) then
		local text = self:Serialize()
		--local text = JsonUtils:safe_encode(self._wrapper)
		if text then
			if saveBinary then
				BinaryEncoder:WriteBytes(self._filePathBin, BinaryEncoder:Encode(CellDataBlockManager.m_binaryEncoder, text))		
			else
				FileUtils:WriteAllText(self._filePath, text)						
			end				
			if generateHeaderFile then
				self:generateHeaderFile()
			end			
			self:debug_print("SUCCESS Saved CellData.. [saveBinary : "..tostring(saveBinary).."]")				
		else
			self:debug_print("FAILED Saving CellData " .. self._name)	
		end	
	end
end


function CellDataBaseWrapper:get_all_blocks()
	local list = {}
	for index, block in pairs(self._wrapper.Blocks) do
		if block.internalEnabled then
			table.insert(list,block)
		end	
	end
	return list
end

function CellDataBaseWrapper:get_block_by_id(ID)
	if ID == 0 then
		return nil
	end
	
	
	return self._blocks_by_IDS[ID]
end

function CellDataBaseWrapper:get_block_by_name(name)
	return self._blocks_by_names[name]
end



function CellDataBaseWrapper:add_block(block, addIndex)
	self:CheckPersistentID(block, false)
	self:EnsureUniqueName(block, self._blocks_by_names, "", true)
	
	if addIndex and addIndex >= 0 then
		table.insert(self._wrapper.Blocks, addIndex, block)
	else
		table.insert(self._wrapper.Blocks, block)
	end
	
	self._blocks_by_names[block.name] = block
	self._blocks_by_IDS[block.persistentID] = block
end


function CellDataBaseWrapper:add_new_block()
	local block = self._block_class:new()
	block.name = "New_Block"
	block.internalEnabled = false
	block.persistentID = 0	
	self:add_block(block, -1)	
	return block
end


function CellDataBaseWrapper:create_new_copy(source)
	local BlocksCount = table.size(self._wrapper.Blocks)
	local index = math.min(table.index_of(self._wrapper.Blocks, source) + 1, BlocksCount)
	
	local t = deep_clone(source)
	t.name = t.name .. "_Copy"
	t.persistentID = 0
	self:CheckPersistentID(t, false)
	if (index == BlocksCount) then
		self:add_block(t, -1)
		return BlocksCount
	end
	
	self:add_block(t, index)
	return index	
end