DramaStateAssault = DramaStateAssault or blt_class(DramaStateBase)
DramaStateAssault.Combat_Chatter_Tension = {
	Min = 50,
	Max = 100
}
DramaStateAssault.Combat_CallOut_Delay = {
	Min = 2,
	Max = 10
}

function DramaStateAssault:setup()
	self._drama_block = self._drama_wrapper:get_block_by_id(5)
	self._shooting_t_check = 2
	self._enter_clbk_id = "DramaStateAssault_Enter"
	self._first_assault_wave = true	
end

function DramaStateAssault:enter(exit_data)
	if self._first_assault_wave then
		self._first_assault_wave = false
	end
	
	self:set_wave_flag(true)
	managers.cell_music:change_state(MusicMachine.MUS_States.Assault)	
	self._last_combat_chatter_t = nil
	if self._machine:last_state_name() == DramaMachine.STATES.Control or self._machine:last_state_name() == DramaMachine.STATES.Anticipation or self._machine:last_state_name() == DramaMachine.STATES.Alert then
		managers.cell_callback:add_delayed_callback(self._enter_clbk_id, math.rand(1,1.25), callback(self,self,"enter_chatter"))	
	end	
end


function DramaStateAssault:enter_chatter()
	if not managers.cell_user:is_cell_host() then
		return
	end
	
	
	local best_canditate = self._drama_manager:get_best_canditate("cell_assault_sli", true, "cell_bot_assault_sli")	
	if best_canditate then
		best_canditate:sound():say("a01x_any",true,true)
	end
end

function DramaStateAssault:check_warcry(t)
	if self:is_owner_alive() then
		local owner = self:owner()
		if not owner:movement():downed() then
			local shooting_t = owner:movement():current_state()._shooting_t
			if shooting_t then
				if t < shooting_t + self._shooting_t_check then
					self:chk_say_player_combat_chatter(t)
				end
			end
		end		
	end
end



function DramaStateAssault:get_additional_tension(max_tension)
	local all_char_criminals = managers.groupai:state():all_char_criminals()
	for u_key, u_data in pairs(all_char_criminals) do
		if u_data and u_data.unit and alive(u_data.unit) and u_data.unit:movement():downed() then
			max_tension = max_tension + 8
		end
	end
	
	if next(managers.trade._criminals_to_respawn) then
		for i, c in ipairs(managers.trade._criminals_to_respawn) do
			max_tension = max_tension + 8
		end
	end
	return max_tension
end

function DramaStateAssault:common_update(t,dt)
		
	if self._drama_manager:go_to_sneaking() or self._drama_manager:go_to_control() then
		self._machine:change_drama_state(DramaMachine.STATES.Alert, true)
		return
	end	
	
	if self._machine:is_locally_owned() then
		self:check_warcry(t)
	end
end

function DramaStateAssault:update(t, dt)	
	self:common_update(t,dt)	
end

function DramaStateAssault:sync_update(t, dt)	
	self:common_update(t,dt)		
end


function DramaStateAssault:chk_say_player_combat_chatter(t)	
	if self._machine:is_seperated_from_teammates() then
		return
	end
	
	
	local current_tension = self._machine:current_tension()
	if current_tension < DramaStateAssault.Combat_Chatter_Tension.Min then
		return
	end
	
	if self._machine:enemies_in_field_of_view() < 2 then
		return
	end
	

	
	
	local tension_val = (math.min(current_tension, DramaStateAssault.Combat_Chatter_Tension.Max) - DramaStateAssault.Combat_Chatter_Tension.Min) / (DramaStateAssault.Combat_Chatter_Tension.Max - DramaStateAssault.Combat_Chatter_Tension.Min)
	if self._last_combat_chatter_t then

		local delay = math.lerp(DramaStateAssault.Combat_CallOut_Delay.Max, DramaStateAssault.Combat_CallOut_Delay.Min, tension_val)
		local delay_t = self._last_combat_chatter_t + delay
		
		if t < delay_t then
			return
		end
		
	end
	
	local chance_tweak = tweak_data.sound.criminal_sound.combat_callout_chance	
	local chance = math.lerp(chance_tweak[1], chance_tweak[2], tension_val)
	local rand = math.random()

	if chance < rand then
		return
	end
	
	self._last_combat_chatter_t = t	
	if managers.cell_options:has_toggled("cell_combat_idle_tog") then
		self:owner():sound():say("g90", true, true)
	end	
end

function DramaStateAssault:debug_prefix()
	return "DramaStateAssault"
end

function DramaStateAssault:idle_dialog()
	managers.player_dialog:queue_dialog(27, {})
end