CellHooks = CellHooks or {}
CellHooks.IntroShowed = false
CellHooks.MenuItems = {}
CellHooks.MenuItems.Toggles = {}
CellHooks.MenuItems.Sliders = {}

CellHooks.MENU_TYPES = {}
CellHooks.MENU_TYPES.TOGGLE = 1
CellHooks.MENU_TYPES.SLIDER = 2

function CellHooks:AddMenuItem(data)
	if data.type == CellHooks.MENU_TYPES.TOGGLE then	
		MenuCallbackHandler[data.callback] = function(self, item)
			if managers.cell_options then
				managers.cell_options:set_toggle_value(data.value_id, item:value() == "on" and true or false)
			end
		end		
		CellHooks.MenuItems.Toggles[data.value_id] = MenuHelper:AddToggle({
			id = data.id,
			title = data.title,
			desc = data.desc,
			callback = data.callback,
			priority = data.priority,
			menu_id = data.menu_id,
			value = data.value,
			show_value = data.show_value,
			localized = data.localized
		})		
	elseif data.type == CellHooks.MENU_TYPES.SLIDER then	
		MenuCallbackHandler[data.callback] = function(self, item)
			if managers.cell_options then
				managers.cell_options:set_slider_value(data.value_id, MathUtils:get_fixed_value(item:value()))
			end
		end		
		CellHooks.MenuItems.Sliders[data.value_id] = MenuHelper:AddSlider({
			id = data.id,
			title = data.title,
			desc = data.desc,
			callback = data.callback,
			value = data.value,
			min = data.min or 0,
			max = data.max or 1,
			step = data.step or 0.01,
			show_value = data.show_value,
			menu_id = data.menu_id,
			priority = data.priority
		})	
	end	
end


Hooks:Add("MenuManagerOnOpenMenu", "MenuManagerOnOpenMenu_Cell", function( menu_manager, menu, position )
	if menu == "menu_main" and not CellHooks.IntroShowed  then
		SoundDevice:set_state("wave_flag", "assault")
		SoundDevice:set_state("acoustic_flag", "acoustic_flat")
		blt.xaudio.setup()
		local sound = CellSoundPlayer:new()			
		CellHooks.IntroShowed = true
		
		local menu_options = {
			[1] = {
				text = "Okay!",
				callback = function()
					log("Stop music!!")
					managers.music:clbk_game_has_music_control(true)
					sound:Stop()
				end,
				is_cancel_button = true
			}
		}
		
		local user_data_block = managers.cell_data:wrapper("UserDataBlock"):get_block_by_id(1)
		local block = managers.cell_data:wrapper("IntroDataBlock"):get_block_by_id(user_data_block.updateLogsBlockID)		
		local description = block.IntroDesc
		if not block.Ignore and not managers.cell_options:has_seen_intro(block.value) then
			QuickMenu:new(managers.localization:text(block.IntroTitle), managers.localization:text(description), menu_options, true)		
			managers.music:clbk_game_has_music_control(false)
			sound:Post("Play_ModIntro",nil,nil)
			managers.cell_options:set_intro_seen(block.value, true)
		end	

		if managers.cell_options:is_obsolete_projects_present() then
			managers.cell_options:_old_projects_detected()
			QuickMenu:new(managers.localization:text("project_cell_obsolete_deadlocke_title"), managers.localization:text("project_cell_obsolete_deadlocke_desc"), menu_options, true)	
		end
		
	end
end)

Hooks:Add('NetworkReceivedData', 'NetworkReceivedData_Cell', function(sender, messageType, data)
	if managers.cell_network then
		if messageType == managers.cell_network:network_id() then
			managers.cell_network:network_receive_data(sender, messageType, data)
		end
	end	
end)

Hooks:Add( "MenuManagerInitialize", "MenuManagerInitialize_Cell", function( menu_manager )
	
	
	function MenuCallbackHandler:clbk_exit_project_cell_menu(item)
		if managers.cell_user and managers.cell_user:has_local_peer() then
			managers.cell_user:local_user():send_cell_values_to_peers()
		end		
	end			
		
end )

Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_Cell", function(loc)
	local path = "assets/Localization/English.txt"
	if FileUtils:PathExists(path) then
		loc:load_localization_file(FileUtils.PATH  .. path, false)			
	end	
end)

Hooks:Add("MenuManagerSetupCustomMenus", "MenuManagerSetupCustomMenus_Cell", function( menu_manager, nodes )
	MenuHelper:NewMenu( "project_cell" )
end)

Hooks:Add("MenuManagerBuildCustomMenus", "MenuManagerBuildCustomMenus_Cell", function(menu_manager, nodes)	
	local menu_data = {}
	menu_data.back_callback = "clbk_exit_project_cell_menu"	
	
	nodes.project_cell = MenuHelper:BuildMenu( "project_cell",  menu_data)
	
	MenuHelper:AddMenuItem( nodes.blt_options, "project_cell", "project_cell_title", "project_cell_desc")	
end)

Hooks:Add("MenuManagerPopulateCustomMenus", "MenuManagerPopulateCustomMenus_Cell", function( menu_manager, nodes )	
			
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_call_tog",
		id = "project_cell_call_tog",
		title = "project_cell_call_tog_title",
		desc = "project_cell_call_tog_desc",
		callback = "clkb_cell_call_tog",
		priority = -1,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	

	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_defend_call_tog",
		id = "project_cell_defend_call_tog",
		title = "project_cell_defend_call_tog_title",
		desc = "project_cell_defend_call_tog_desc",
		callback = "clkb_cell_defend_call_tog",
		priority = -2,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})

	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_combat_idle_tog",
		id = "project_cell_combat_idle_tog",
		title = "project_cell_combat_idle_tog_title",
		desc = "project_cell_combat_idle_tog_desc",
		callback = "clkb_cell_combat_idle_tog",
		priority = -3,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})

	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_downed_help_tog",
		id = "project_cell_downed_help_tog",
		title = "project_cell_downed_help_tog_title",
		desc = "project_cell_downed_help_tog_desc",
		callback = "clkb_cell_downed_help_tog",
		priority = -4,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_revive_talk_tog",
		id = "project_cell_revive_talk_tog",
		title = "project_cell_revive_talk_tog_title",
		desc = "project_cell_revive_talk_tog_desc",
		callback = "clkb_cell_revive_talk_tog",
		priority = -4.1,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_react_downed_tog",
		id = "project_cell_react_downed_tog",
		title = "project_cell_react_downed_tog_title",
		desc = "project_cell_react_downed_tog_desc",
		callback = "clbk_cell_react_downed_tog",
		priority = -4.2,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_narrator_pager_tog",
		id = "project_cell_narrator_pager_tog",
		title = "project_cell_narrator_pager_tog_title",
		desc = "project_cell_narrator_pager_tog_desc",
		callback = "clbk_cell_narrator_pager_tog",
		priority = -4.3,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_tie_civilian_tog",
		id = "project_cell_tie_civilian_tog",
		title = "project_cell_tie_civilian_tog_title",
		desc = "project_cell_tie_civilian_tog_desc",
		callback = "clbk_cell_tie_civilian_tog_tog",
		priority = -4.4,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_hostage_convert_tog",
		id = "project_cell_hostage_convert_tog",
		title = "project_cell_hostage_convert_tog_title",
		desc = "project_cell_hostage_convert_tog_desc",
		callback = "clbk_cell_hostage_convert_tog",
		priority = -4.5,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_revived_tog",
		id = "project_cell_revived_tog",
		title = "project_cell_revived_tog_title",
		desc = "project_cell_revived_tog_desc",
		callback = "clbk_cell_revived_tog",
		priority = -4.6,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.TOGGLE, 
		value_id = "cell_flash_bang_tog",
		id = "project_cell_flash_bang_tog",
		title = "project_cell_flash_bang_tog_title",
		desc = "project_cell_flash_bang_tog_desc",
		callback = "clbk_cell_flash_bang_tog",
		priority = -4.7,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	
	
	
	MenuHelper:AddDivider({
		id = "project_cell_divide_sliders",
		size = 20,
		menu_id = "project_cell",
		priority = -5,
	})
	
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.SLIDER, 
		value_id = "cell_anticipation_sli",
		id = "project_cell_anticipation_sli",
		title = "project_cell_anticipation_sli_title",
		desc = "project_cell_anticipation_sli_desc",
		callback = "clbk_set_anticipation_sli",
		priority = -6,
		menu_id = "project_cell",
		value = false,
		show_value = true,
		localized = true
	})
	

	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.SLIDER, 
		value_id = "cell_assault_sli",
		id = "project_cell_assault_sli",
		title = "project_cell_assault_sli_title",
		desc = "project_cell_assault_sli_desc",
		callback = "clbk_set_assault_sli",
		value = 1,
		min = 0,
		max = 1,
		step = 0.01,
		show_value = true,
		menu_id = "project_cell",
		priority = -7
	})
	

	

	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.SLIDER, 
		value_id = "cell_post_assault_sli",
		id = "project_cell_post_assault_sli",
		title = "project_cell_post_assault_sli_title",
		desc = "project_cell_post_assault_sli_desc",
		callback = "clbk_set_post_assault_sli",
		value = 1,
		min = 0,
		max = 1,
		step = 0.01,
		show_value = true,
		menu_id = "project_cell",
		priority = -8
	})
	
	
	MenuHelper:AddDivider({
		id = "project_cell_divide_bots",
		size = 20,
		menu_id = "project_cell",
		priority = -9,
	})

	
	--- BOTS
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.SLIDER, 
		value_id = "cell_bot_anticipation_sli",
		id = "project_cell_bot_anticipation_sli",
		title = "project_cell_bot_anticipation_sli_title",
		desc = "project_cell_bot_anticipation_sli_desc",
		callback = "clbk_set_bot_anticipation_sli",
		value = 1,
		min = 0,
		max = 1,
		step = 0.01,
		show_value = true,
		menu_id = "project_cell",
		priority = -10
	})
	
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.SLIDER, 
		value_id = "cell_bot_assault_sli",
		id = "project_cell_bot_assault_sli",
		title = "project_cell_bot_assault_sli_title",
		desc = "project_cell_bot_assault_sli_desc",
		callback = "clbk_set_bot_assault_sli",
		value = -1,
		min = 0,
		max = 1,
		step = 0.01,
		show_value = true,
		menu_id = "project_cell",
		priority = -11
	})
	
	CellHooks:AddMenuItem({
		type = CellHooks.MENU_TYPES.SLIDER, 
		value_id = "cell_bot_post_assault_sli",
		id = "project_cell_bot_post_assault_sli",
		title = "project_cell_bot_post_assault_sli_title",
		desc = "project_cell_bot_post_assault_sli_desc",
		callback = "clbk_set_bot_post_assault_sli",
		value = 1,
		min = 0,
		max = 1,
		step = 0.01,
		show_value = true,
		menu_id = "project_cell",
		priority = -12
	})

	


	
end)