dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_Base.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_Silence.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_Sneaking.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_Control.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_Anticipation.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_Assault.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_FakeAssault.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_Escape.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_AlessoBase.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_AlessoSneaking.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_AlessoControl.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_AlessoAnticipation.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_AlessoAssault.lua")
dofile(ModPath .. "Lua/Scripts/MusicStates/MUS_AlessoFakeAssault.lua")

MusicMachine = MusicMachine or blt_class(StateMachine)
MusicMachine.MusicEnabled = true
MusicMachine.MUS_States = {}
MusicMachine.MUS_States.None = "none"
MusicMachine.MUS_States.Silence = "silence"
MusicMachine.MUS_States.Sneaking = "sneaking"
MusicMachine.MUS_States.Control = "control"
MusicMachine.MUS_States.Anticipation = "anticipation"
MusicMachine.MUS_States.Assault = "assault"
MusicMachine.MUS_States.FakeAssault = "fake_assault"
MusicMachine.MUS_States.Escape = "escape"
MusicMachine.MUS_States.Alesso = "alesso"
MusicMachine.MUS_States.AlessoSneaking = "alesso_sneaking"
MusicMachine.MUS_States.AlessoControl = "alesso_control"
MusicMachine.MUS_States.AlessoAnticipation = "alesso_anticipation"
MusicMachine.MUS_States.AlessoAssault = "alesso_assault"
MusicMachine.MUS_States.AlessoFakeAssault = "alesso_fake_assault"

function MusicMachine:init(music_manager)
	MusicMachine.super.init(self, music_manager)
	self._music_manager = music_manager
	self._state_overrides = {}
	self:_setup_states()
end

function MusicMachine:_setup_states()
	self:enable_debug(false)
	
	self._mus_base = MUS_Base:new()	
	self._mus_sneaking = MUS_Sneaking:new()
	self._mus_control = MUS_Control:new()
	self._mus_anticipation = MUS_Anticipation:new()
	self._mus_assault = MUS_Assault:new()
	self._mus_fake_assault = MUS_FakeAssault:new()	
	self._mus_escape = MUS_Escape:new()	
	self._mus_alesso = MUS_AlessoBase:new()
	self._mus_alesso_sneaking = MUS_AlessoSneaking:new()
	self._mus_alesso_control = MUS_AlessoControl:new()
	self._mus_alesso_anticipation = MUS_AlessoAnticipation:new()
	self._mus_alesso_assault = MUS_AlessoAssault:new()
	self._mus_alesso_fake_assault = MUS_AlessoFakeAssault:new()
	
			
	
	local level_id = Global.level_data and Global.level_data.level_id
	if level_id == "arena" then
		self:_setup_alesso()
	elseif tweak_data.levels[level_id].music == "ghost" then
		self:_setup_ghost()
	else
		self:_setup_heist()
	end

	
	self:reset()
end

function MusicMachine:_setup_heist()
	self._start_state = self._mus_base
	self:add_state(MusicMachine.MUS_States.None, self._mus_base)	
	self:add_state(MusicMachine.MUS_States.Silence, MUS_Silence:new())	
	self:add_state(MusicMachine.MUS_States.Sneaking, self._mus_sneaking)
	self:add_state(MusicMachine.MUS_States.Control, self._mus_control)
	self:add_state(MusicMachine.MUS_States.Anticipation, self._mus_anticipation)
	self:add_state(MusicMachine.MUS_States.Assault, self._mus_assault)
	self:add_state(MusicMachine.MUS_States.FakeAssault, self._mus_fake_assault)	
	self:add_state(MusicMachine.MUS_States.Escape, self._mus_escape)
	self:add_state(MusicMachine.MUS_States.Alesso, self._mus_alesso)
	self:add_state(MusicMachine.MUS_States.AlessoSneaking, self._mus_alesso_sneaking)
	self:add_state(MusicMachine.MUS_States.AlessoControl, self._mus_alesso_control)
	self:add_state(MusicMachine.MUS_States.AlessoAnticipation, self._mus_alesso_anticipation)
	self:add_state(MusicMachine.MUS_States.AlessoAssault, self._mus_alesso_assault)
	self:add_state(MusicMachine.MUS_States.AlessoFakeAssault, self._mus_alesso_fake_assault)
end

function MusicMachine:_setup_alesso()
	self._start_state  = self._mus_alesso
	self:add_state(MusicMachine.MUS_States.None, self._mus_base)	
	self:add_state(MusicMachine.MUS_States.Silence, MUS_Silence:new())	
	self:add_state(MusicMachine.MUS_States.Sneaking, self._mus_alesso_sneaking)
	self:add_state(MusicMachine.MUS_States.Control, self._mus_alesso_control)
	self:add_state(MusicMachine.MUS_States.Anticipation, self._mus_alesso_anticipation)
	self:add_state(MusicMachine.MUS_States.Assault, self._mus_alesso_assault)
	self:add_state(MusicMachine.MUS_States.FakeAssault, self._mus_alesso_fake_assault)	
	self:add_state(MusicMachine.MUS_States.Escape, self._mus_escape)
	self:add_state(MusicMachine.MUS_States.Alesso, self._mus_alesso)
	self:add_state(MusicMachine.MUS_States.AlessoSneaking, self._mus_alesso_sneaking)
	self:add_state(MusicMachine.MUS_States.AlessoControl, self._mus_alesso_control)
	self:add_state(MusicMachine.MUS_States.AlessoAnticipation, self._mus_alesso_anticipation)
	self:add_state(MusicMachine.MUS_States.AlessoAssault, self._mus_alesso_assault)
	self:add_state(MusicMachine.MUS_States.AlessoFakeAssault, self._mus_alesso_fake_assault)
end

function MusicMachine:_setup_ghost()
	self._start_state = self._mus_base
	self:add_state(MusicMachine.MUS_States.None, self._mus_base)	
	self:add_state(MusicMachine.MUS_States.Silence, MUS_Silence:new())	
	self:add_state(MusicMachine.MUS_States.Sneaking, self._mus_base)
	self:add_state(MusicMachine.MUS_States.Control, self._mus_base)
	self:add_state(MusicMachine.MUS_States.Anticipation, self._mus_base)
	self:add_state(MusicMachine.MUS_States.Assault, self._mus_base)
	self:add_state(MusicMachine.MUS_States.FakeAssault, self._mus_base)	
	self:add_state(MusicMachine.MUS_States.Escape, self._mus_escape)
	self:add_state(MusicMachine.MUS_States.Alesso, self._mus_base)
	self:add_state(MusicMachine.MUS_States.AlessoSneaking, self._mus_base)
	self:add_state(MusicMachine.MUS_States.AlessoControl, self._mus_base)
	self:add_state(MusicMachine.MUS_States.AlessoAnticipation, self._mus_base)
	self:add_state(MusicMachine.MUS_States.AlessoAssault, self._mus_base)
	self:add_state(MusicMachine.MUS_States.AlessoFakeAssault, self._mus_base)
end


function MusicMachine:_change_state(mus_state)
	if self._current_state == mus_state then
		if self._debug_enabled  then
			log("[MusicMachine:_change_state] new mus_state is the same, not changing")
		end
		return
	end
	
	if MusicMachine.MusicEnabled then
		MusicMachine.super._change_state(self, mus_state)
	end
end