CellDataBlockManager = CellDataBlockManager or blt_class()
CellDataBlockManager.DATABLOCKS_RESOURCE_PATH = "assets/Resources/"
CellDataBlockManager.DATABLOCKS_PATH = "assets/CellData/"
CellDataBlockManager.DATABLOCKS_HEADER_PATH = "Lua/Scripts/CellData/Headers/"
CellDataBlockManager.CELLDATA_WRAPPER_PATH = "Lua/Scripts/CellData/CellDataWrappers/"
CellDataBlockManager.CELLDATA_SCRIPT_PATH = "Lua/Scripts/CellData/"

CellDataBlockManager.IS_RESOURCE_FILE = false
CellDataBlockManager.SAVE_ON_STARTUP = false
CellDataBlockManager.SAVE_BINARY = false
CellDataBlockManager.GENERATE_HEADERS = false

CellDataBlockManager.LOAD_BINARY = false


CellDataBlockManager.m_binaryEncoder = nil
CellDataBlockManager.DEBUG_ENABLED = true
CellDataBlockManager.ALL_DATABLOCK_TYPES = {
	"StateDataBlock",
	"PlayerDialogStoryDataBlock",
	"PlayerModOptionsDataBlock",
	"SoundEventDataBlock",
	"SoundObjectDataBlock",
	"PlayerVoiceInteractionsDataBlock",
	"CharacterTweaksDataBlock",
	"CharacterDataBlock",
	"SwitchContainerDataBlock",
	"DramaStateDataBlock",
	"UserDataBlock",
	"IntroDataBlock",
	"PlayerDataBlock",
	"PlayerDialogDataBlock",
	"MusicStateDataBlock",
	"PlayerDialogSegmentGroupDataBlock",
	"VanillaDialogDataBlock"
}


function CellDataBlockManager:debug_print(txt)
	if CellDataBlockManager.DEBUG_ENABLED then
		log("CellDataBlockManager - " .. txt)
	end	
end


function CellDataBlockManager:init()	
	self._save_on_startup = CellDataBlockManager.SAVE_ON_STARTUP
	self._save_binary = CellDataBlockManager.SAVE_BINARY
	self._generate_headers = CellDataBlockManager.GENERATE_HEADERS
	FileUtils:LoadFile(CellDataBlockManager.CELLDATA_SCRIPT_PATH .."CellDataHeader.lua");
	FileUtils:LoadFile(CellDataBlockManager.CELLDATA_SCRIPT_PATH .."CellDataBlockBase.lua");
	FileUtils:LoadFile(CellDataBlockManager.CELLDATA_WRAPPER_PATH.."CellDataBaseWrapper.lua")
	CellDataBlockManager.m_binaryEncoder = BP_Coder:new(672683)
	self:setup_all_blocks()
end 




function CellDataBlockManager:SaveAllDataBlocksToDisk(generateHeaderFile, saveBinary)
	for _, wrapper in pairs(self._block_wrappers) do
		if wrapper then
			wrapper:DoSaveToDisk(generateHeaderFile, saveBinary)
		end
	end
end


function CellDataBlockManager:setup_all_blocks()
	self._block_wrappers = {}
	
	if not FileUtils:DirectoryExists(CellDataBlockManager.DATABLOCKS_PATH) then
		error(CellDataBlockManager.DATABLOCKS_PATH .. " does not exist! Are we missing data blocks directory?")
	end
	
	
	local wrapper_types = CellDataBlockManager.ALL_DATABLOCK_TYPES
	if wrapper_types then
		for index, name in pairs(wrapper_types) do
			self:setup_wrapper(name)			
		end			
	end	

	if self._save_on_startup then
		self:SaveAllDataBlocksToDisk(self._generate_headers, self._save_binary)
	end
end



function CellDataBlockManager:setup_wrapper(wrapperName)		
	local customWrapperName = wrapperName:gsub("DataBlock","DataWrapper");	
	FileUtils:LoadFile(CellDataBlockManager.CELLDATA_WRAPPER_PATH ..customWrapperName..".lua");
	local customWrapper = rawget(_G, customWrapperName)		
	if (customWrapper) then
		self:debug_print("Loading customWrapper: "..customWrapperName)
		self._block_wrappers[wrapperName] = customWrapper:new(wrapperName, CellDataBlockManager.DEBUG_ENABLED)	
		return self._block_wrappers[wrapperName]
	else
		self:debug_print("customWrapper: "..customWrapperName.." doesn't exist! Loading Base Wrapper")
		self._block_wrappers[wrapperName] = CellDataBaseWrapper:new(wrapperName,  CellDataBlockManager.DEBUG_ENABLED)	
		return self._block_wrappers[wrapperName]
	end
	
	return nil	
end



function CellDataBlockManager:wrapper(wrapperName)
	if not self._block_wrappers[wrapperName] then
		error("[CellDataBlockManager:wrapper] wrapperName "..tostring(wrapperName).." not valid!!")
		return
	end
	return self._block_wrappers[wrapperName]
end

