CellNetworkManager = CellNetworkManager or blt_class()
CellNetworkManager.DEBUG_ENABLED = true
function CellNetworkManager:init()
	self._networkID = "ProjCell"
	self._networkVersion = managers.cell_data:wrapper("UserDataBlock"):get_block_by_id(1).networkVersion
	self._action_validations = {}
end

function CellNetworkManager:version()
	return self._networkVersion
end

function CellNetworkManager:network_id()
	return self._networkID
end

function CellNetworkManager:debug_print(txt)
	if CellNetworkManager.DEBUG_ENABLED then
		log("CellNetworkManager - " .. txt)
	end
end

function CellNetworkManager:network_receive_data(sender_peer_id, messageType, data)
	if messageType == self._networkID then
		self:execute_packet_action(sender_peer_id, data)	
	end	
end


function CellNetworkManager:test_network_received_self(messageType, data)
	local str = JsonUtils:safe_encode(data)
	self:network_receive_data(managers.cell_user:local_peer_id(), messageType, str)
end


function CellNetworkManager:register_action_validation(validation)
	local packet_id = validation:packet_id()
	if self._action_validations[packet_id] then	
		self:debug_print("ERROR : packet_id already registered : " .. tostring(packet_id))
		return
	end
	
	self._action_validations[packet_id] = validation
end

function CellNetworkManager:execute_packet_action(sender_peer_id, str)
	local data = JsonUtils:safe_decode(str)
	
	if not data then
		return
	end
	
	if data[1] == nil or type(data[1]) ~= "string" then
		self:debug_print("No packet id specified from peer id : " .. tostring(sender_peer_id))
		return
	end
	
	if data[2] == nil then
		self:debug_print("packet has returned nil from peer id : " .. tostring(sender_peer_id))
		return
	end	
	
	local validation = self._action_validations[data[1]]
	if validation then
		validation:Do(data[2], sender_peer_id)
	end
end