CellOptionsManager = CellOptionsManager or blt_class()
CellOptionsManager.ModPath = ModPath
CellOptionsManager.SavePath = SavePath
CellOptionsManager.DEBUG_ENABLED = true
CellOptionsManager.TOGGLES = {
	"cell_speech_tog",
	"cell_call_tog",
	"cell_defend_call_tog",
	"cell_combat_idle_tog",
	"cell_downed_help_tog",
	"cell_react_downed_tog",
	"cell_revive_talk_tog",
	"cell_narrator_pager_tog",
	"cell_hostage_convert_tog",
	"cell_tie_civilian_tog",
	"cell_revived_tog",
	"cell_flash_bang_tog"
}
CellOptionsManager.SLIDER_VALUES = {
	"cell_assault_sli",
	"cell_bot_assault_sli",
	"cell_speech_sli",
	"cell_anticipation_sli",
	"cell_bot_anticipation_sli",
	"cell_post_assault_sli",
	"cell_bot_post_assault_sli"
}

CellOptionsManager.INTROS = {
	"cell_beta_intro"
}

function CellOptionsManager:get_default_data()
	return {
		toggles = {},
		sliders = {},
		intros = {}
	}
end

function CellOptionsManager:is_obsolete_projects_present()
	return false
end

function CellOptionsManager:init()
	self._user_data = self:get_default_data()
	self._path = CellOptionsManager.ModPath
	self._save_path = CellOptionsManager.SavePath .. "CellSaveData.txt"
	self:load_user_settings()
	self:_setup_menu_nodes()
end

function CellOptionsManager:_setup_menu_nodes()
	for name, item in pairs(CellHooks.MenuItems.Toggles) do
		item:set_value(self:has_toggled(name) and "on" or "off")
	end	
	for name, item in pairs(CellHooks.MenuItems.Sliders) do
		item:set_value(self:slider_value(name))
	end	
	self:debug_print("Successfully modified the menu items based on the user settings..")
	if self:is_obsolete_projects_present() then
		self:_old_projects_detected()
	end
end


function CellOptionsManager:_old_projects_detected()
	for name, item in pairs(CellHooks.MenuItems.Toggles) do
		item:set_value("off")
		item:set_enabled(false)
	end	
	for name, item in pairs(CellHooks.MenuItems.Sliders) do
		item:set_enabled(false)
	end	
end


function CellOptionsManager:get_save_data()
	local save_data = self:get_default_data()	
	
	for _, value_name in pairs(CellOptionsManager.TOGGLES) do
		save_data.toggles[value_name] = self:has_toggled(value_name)
	end	
	
	for _, slider_name in pairs(CellOptionsManager.SLIDER_VALUES) do
		save_data.sliders[slider_name] = self:slider_value(slider_name)
	end	
	
	for _, intro_name in pairs(CellOptionsManager.INTROS) do
		save_data.intros[intro_name] = self:has_seen_intro(intro_name)
	end	
	
	return save_data
end


function CellOptionsManager:save_user_settings()
	local save_data = self:get_save_data()
	local file = io.open( self._save_path, "w+" )
	if file then
		local json_string = JsonUtils:safe_encode( save_data )		
		file:write(json_string)
		file:close()		
		--self:debug_print("Saved user data..")
	end
end

function CellOptionsManager:debug_print(txt)
	if CellOptionsManager.DEBUG_ENABLED then
		log("CellOptionsManager - " .. txt)
	end
end

function CellOptionsManager:load_user_settings()
	local file = io.open( self._save_path, "r" )
	if file then	
		local json_string = file:read("*all")
		self._user_data = JsonUtils:safe_decode(json_string or {}) or {}		
		file:close()
		self:debug_print("Cell user data succefully loaded.")		
	else
		self:debug_print("Cell user data didn't exist, creating a new one.")		
	end
	self:clean_load_settings()
end

function CellOptionsManager:clean_load_settings()
	local clean_save = false
	if not self._user_data or type(self._user_data) ~= "table" then
		clean_save = true
		self._user_data = {}
		self:debug_print("ERROR : user data is corrupted, cleaning it up!!")	
	end
	if not self._user_data.toggles or type(self._user_data.toggles) ~= "table" then
		clean_save = true
		self._user_data.toggles = {}
		self:debug_print("ERROR : user data for toggles is corrupted!!, cleaning it up!!")	
	end
	if not self._user_data.sliders or type(self._user_data.sliders) ~= "table" then
		clean_save = true
		self._user_data.sliders = {}
		self:debug_print("ERROR : user data for sliders is corrupted!!, cleaning it up!!")	
	end
	
	if not self._user_data.intros or type(self._user_data.intros) ~= "table" then
		clean_save = true
		self._user_data.intros = {}
		self:debug_print("ERROR : user data for intros is corrupted!!, cleaning it up!!")	
	end
	
	if clean_save then
		self:save_user_settings()
	end	
end

function CellOptionsManager:set_toggle_value(value_name, enabled)
	if not value_name or type(value_name) ~= "string" then
		return
	end
	
	if type(enabled) ~= "boolean" then
		return
	end
	
	if not table.contains(CellOptionsManager.TOGGLES, value_name) then
		self:debug_print("ERROR : toggle name " .. tostring(value_name) .. " is invalid!")
		return
	end
	
	self._user_data.toggles[value_name] = enabled
	self:save_user_settings()
end

function CellOptionsManager:has_toggled(value_name)
	return self._user_data.toggles[value_name] and true or false
end

function CellOptionsManager:slider_value(slider_name)
	return self._user_data.sliders[slider_name] or 0
end

function CellOptionsManager:set_slider_value(slider_name, value)
	if not slider_name or type(slider_name) ~= "string" then
		return
	end
	
	if not value or type(value) ~= "number" then
		return
	end
	
	if not table.contains(CellOptionsManager.SLIDER_VALUES, slider_name) then
		self:debug_print("ERROR : slider name " .. tostring(slider_name) .. " is invalid!")
		return
	end
	
	self._user_data.sliders[slider_name] = value
	self:save_user_settings()
end

function CellOptionsManager:set_intro_seen(intro_name, enabled)
	if not intro_name or type(intro_name) ~= "string" then
		return
	end
	
	if type(enabled) ~= "boolean" then
		return
	end
	
	if not table.contains(CellOptionsManager.INTROS, intro_name) then
		self:debug_print("ERROR : intro name " .. tostring(intro_name) .. " is invalid!")
		return
	end
	
	self._user_data.intros[intro_name] = enabled
	self:save_user_settings()
end

function CellOptionsManager:has_seen_intro(intro_name)
	return self._user_data.intros[intro_name] and true or false
end