dofile(ModPath.."Lua/Scripts/CellUsers/LocalCellUser.lua")
dofile(ModPath.."Lua/Scripts/CellUsers/HuskCellUser.lua")

CellUserManager = CellUserManager or blt_class()
CellUserManager.DEBUG_ENABLED = true
function CellUserManager:init()
	self._users = {}
	self._debug_enabled = CellUserManager.DEBUG_ENABLED
	self._user_joined_callbacks = {}
	self._user_left_callbacks = {}
	self._cell_host_callbacks = {}
	self._is_local_peer_dirty = false
	self._local_dirty_peer = nil
	self._local_user = nil
	self._has_local_peer = false
	self._queue_request = false
	self._beginner = false
	self:_setup_cell_net_validations()	
end

function CellUserManager:_setup_cell_net_validations()
	self._net_action = CellNetValidationAction:new("User", callback(self,self,"do_sync_cell_values"))	
	self._verify_action = CellNetValidationAction:new("Vali", callback(self,self,"do_validation"))	
	self._received_action = CellNetValidationAction:new("Regi", callback(self,self,"do_register"))	
end

function CellUserManager:do_validation(data, sender_id)
	self:do_register(data, sender_id)
	self:send_handshake(sender_id)
end

function CellUserManager:do_register(data, sender_id)
	if self:is_peer_registered(sender_id) then
		self:debug_print("Sender ID " .. tostring(sender_id) .. " has been registered already!")
		return
	end
	
	local current_version = managers.cell_network:version()
	if data.v ~= current_version then
		self:debug_print("Cell User's version mismatch! Current Version : " .. tostring(current_version) .. " Cell User's Version : " .. tostring(data.v))
		return
	end
	
	local new_cell_user = self:register_user_peer_id(sender_id, data.b and true or false)	
	if data.c then
		new_cell_user:sync_cell_values(data.c)	
	end
end

function CellUserManager:get_entry_data()
	local data = {}	
	data.b = self._beginner and true or false	
	data.c = self._local_user:get_cell_values_data()
	data.v = managers.cell_network:version()
	return data
end

function CellUserManager:send_request()
	self._verify_action:send_to_peers(self:get_entry_data())	
end

function CellUserManager:send_handshake(peer_id)	
	self._received_action:send_to_peer(peer_id, self:get_entry_data())	
end

function CellUserManager:net_action()
	return self._net_action
end


function CellUserManager:get_cell_user_by_peer_id(peer_id)
	return self._users[peer_id]
end

function CellUserManager:all_cell_users()
	return self._users
end

function CellUserManager:get_cell_user_by_unit(unit)
	for id, cell_user in pairs(self._users) do
		if cell_user:unit() == unit then
			return cell_user
		end
	end	
	return nil
end

function CellUserManager:get_cell_user_by_character(character)
	for id, cell_user in pairs(self._users) do
		if cell_user:character_name() == character then
			return cell_user
		end
	end
	
	return nil
end

function CellUserManager:do_sync_cell_values(data, sender)
	local cell_user = self._users[sender] 
	
	if not cell_user then
		self:debug_print("ERROR : Peer id " .. tostring(sender) .. " is not registered as a cell user!!")
		return
	end
	
	cell_user:sync_cell_values(data)	
end


function CellUserManager:attempt_to_send_request()
	if self._has_local_peer then
		self:send_request()
		return
	end	
	self:queue_request()
end

function CellUserManager:update_queue_request()
	if not self._queue_request then
		return
	end
	
	if not self._has_local_peer then
		return
	end
	
	self:debug_print("Request sent, via queueing...")	
	self:send_request()	
	self._queue_request = false
end

function CellUserManager:queue_request()
	self:debug_print("[BaseNetworkSession:on_load_complete] Tried to send a request to peers without a local peer, queueing the request..")
	self._queue_request = true
end

function CellUserManager:update(t,dt)
	self:update_dirty_local_peer()
	self:update_queue_request()
end

function CellUserManager:register_user_peer_id(peer_id, is_beginner)
	return self:register_user(managers.network:session():peer(peer_id), false, is_beginner)
end

function CellUserManager:is_peer_registered(peer_id)
	return self._users[peer_id] and true or false
end

function CellUserManager:set_beginner(enable)
	self._beginner = enable
	for i, user in pairs(self._users) do
		if user then
			user:start_from_beginning()
		end		
	end
end

function CellUserManager:is_beginner()
	return self._beginner
end

function CellUserManager:update_dirty_local_peer()
	if self._local_dirty_peer == nil then
		return
	end
	
	if self._local_dirty_peer:id() > 0 then
		self:register_user(self._local_dirty_peer, true, false)
		self._local_dirty_peer = nil
	end
end


function CellUserManager:register_local_peer(peer)
	if peer:id() <= 0 then
		self:debug_print("Queuing local dirty peer : " .. tostring(peer:name()))
		self._local_dirty_peer = peer
	else
		self:register_user(peer, true, false)
	end
end

function CellUserManager:register_user(peer, is_local, is_beginner)
	is_beginner = is_beginner and true or false
	local peer_id = peer:id()
	local new_user
	if is_local then
		new_user = LocalCellUser:new(peer)
		self._local_user = new_user		
		self._has_local_peer = true
		self:debug_print("Registered Local CellUser : " .. peer:name() .. " with peer id " .. tostring(peer_id) .. " is beginner " .. tostring(is_beginner))
	else
		new_user = HuskCellUser:new(peer)
		self:debug_print("Registered Husk CellUser : " .. peer:name() .. " with peer id " .. tostring(peer_id) .. " is beginner " .. tostring(is_beginner))
	end
	
	if is_beginner then
		new_user:start_from_beginning()
	end	
	
	self._users[peer_id] = new_user
	
	self:on_peer_joined(peer_id)
	
	self:determine_cell_host()
	
	return new_user
end

function CellUserManager:local_user()
	return self._local_user
end

function CellUserManager:local_peer_id()
	return self._local_user:peer_id()
end

function CellUserManager:add_join_callback(clbk)
	table.insert(self._user_joined_callbacks, clbk)
end

function CellUserManager:add_left_callback(clbk)
	table.insert(self._user_left_callbacks, clbk)
end

function CellUserManager:add_cell_host_callback(clbk)
	table.insert(self._cell_host_callbacks, clbk)
end

function CellUserManager:on_peer_joined(peer_id)
	for _, clbk in pairs(self._user_joined_callbacks) do
		if clbk then
			clbk(peer_id)
		end
	end
end

function CellUserManager:on_peer_left(peer_id)
	for _, clbk in pairs(self._user_left_callbacks) do
		if clbk then
			clbk(peer_id)
		end
	end
end

function CellUserManager:on_cell_host(enabled)
	for _, clbk in pairs(self._cell_host_callbacks) do
		if clbk then
			clbk(enabled)
		end
	end
end

function CellUserManager:unregister_user(peer_id)
	self:debug_print("Unregisterd user with the peer_id "..tostring(peer_id))
	self._users[peer_id] = nil	
	self:on_peer_left(peer_id)
	self:determine_cell_host()
end

function CellUserManager:determine_cell_host()
	local priority_user_id, priority_user
	
	local has_beginner = false
	for i = 1, 2 do
		local check_beginner = i == 1
		for i, user in pairs(self._users) do
			if user then
				if not priority_user or priority_user_id > user:peer_id() then
					local is_beginner = user:has_started_from_beginning()
					if not check_beginner or is_beginner then
						priority_user = user
						priority_user_id = user:peer_id()
						has_beginner = check_beginner and is_beginner
					end					
				end
			end		
		end
		
		if has_beginner then
			break;
		end
	end
	
	
	if not priority_user then
		self:debug_print("No priority user found to be cell host!")
		return
	end
	
	self._cell_host = priority_user	
	
	local is_cell_host_local = self._cell_host:is_local_user()
	
	self:on_cell_host(is_cell_host_local)	
	
	if is_cell_host_local then
		if self._cell_host:has_started_from_beginning() then
			self:debug_print("Local user is now Cell Host, that has started from the beginning")
		else
			self:debug_print("Local user is now Cell Host")
		end
		
	else
		if self._cell_host:has_started_from_beginning() then
			self:debug_print("Husk user by the name of "..self._cell_host:name().. " is now the Cell Host, that has started from the beginning")
		else
			self:debug_print("Husk user by the name of "..self._cell_host:name().. " is now the Cell Host")
		end
		
	end
end

function CellUserManager:is_cell_host()
	return self._cell_host:is_local_user()
end

function CellUserManager:cell_host_peer_id()
	return self._cell_host:peer_id()
end

function CellUserManager:has_local_peer()
	return self._has_local_peer
end


function CellUserManager:debug_print(txt)
	if not self._debug_enabled then
		return
	end
	
	log("CellUserManager - " .. txt)
end