dofile(ModPath.."Lua/Scripts/Machines/DramaMachine.lua")
dofile(ModPath.."Lua/Scripts/DramaStates/DramaStateFilterBase.lua")
dofile(ModPath.."Lua/Scripts/DramaStates/DramaStateFilterClient.lua")
dofile(ModPath.."Lua/Scripts/DramaStates/DramaStateFilterForced.lua")


DramaManager = DramaManager or blt_class()
DramaManager.DEBUG_ENABLED = true
function DramaManager:init()
	self._debug_enabled = DramaManager.DEBUG_ENABLED
	self._has_forced_state = false
	self._revive_line_cooldown_t = nil	
	self._t = 0
	self._filtered_state_host = DramaStateFilterBase:new()		
	self._filtered_state_client = DramaStateFilterClient:new()		
	self._filtered_state_forced = DramaStateFilterForced:new()
	self._filtered_state_default = self._filtered_state_host	
	self._current_state_filtered = self._filtered_state_default		
	
	
	
	self:_setup_network()
	self:_setup_cell_host_clbks()
	self:_setup_listener()
	self:_reset_synced_states()	
end

function DramaManager:_setup_network()
	self._net_action = CellNetValidationAction:new("DRA", callback(self,self,"do_sync_state"))	
end

function DramaManager:_setup_cell_host_clbks()
	managers.cell_user:add_cell_host_callback(callback(self,self,"on_cell_host"))
end

function DramaManager:_setup_listener()
	self._criminal_added_key = "DramaManager_CriminalAdded"
	self._criminal_removed_key = "DramaManager_CriminalRemove"
	managers.criminals:add_listener(self._criminal_added_key, {"on_criminal_added"}, callback(self, self, "on_criminal_added"))		
	managers.criminals:add_listener(self._criminal_removed_key, {"on_criminal_removed"}, callback(self, self, "on_criminal_removed"))		
end


function DramaManager:on_criminal_added(name, unit, peer_id, ai)
	if unit == nil then
		self:debug_print("ERROR : Character " .. tostring(name) .. " unit has returned nil!!")
		return 
	end
	
	
	local machine = DramaMachine:new({
		drama_manager = self,
		unit = unit,
		name = name,
		ai = ai
	})
	
	self._synced_machines[name] = machine
	if unit:base().is_local_player then
		self._local_machine = machine
		self:debug_print("Adding a Local DramaMachine for character : " .. tostring(name))		
	else
		self:debug_print("Adding a Synced DramaMachine for character : " .. tostring(name))		
	end	
end

function DramaManager:on_cell_host(enabled)
	self._filtered_state_default = enabled and self._filtered_state_host or self._filtered_state_client
end

function DramaManager:synced_machines()
	return self._synced_machines
end

function DramaManager:on_criminal_removed(char_data)
	if not self._synced_machines[char_data.name] then
		self:debug_print("ERROR :  criminal machine " .. tostring(char_data.name) .. " does not exist!!")
		return	
	end
	
	self:debug_print("Removing DramaMachine from " .. tostring(char_data.name) )
	
	if self._local_machine and self._local_machine:character_name() == char_data.name then
		self._local_machine = nil
	end	
	
	self._synced_machines[char_data.name]:unregister()
end

function DramaManager:debug_print(txt)
	if self._debug_enabled then
		log("DramaManager - " .. txt)
	end
end


function DramaManager:_reset_synced_states()	
	self._synced_machines = {}
	self._synced_tensions = {}
	self._peer_synced_states = {}
end

function DramaManager:reset_local_state()	
	self:change_local_state(DramaMachine.STATES.Idle)
end

function DramaManager:local_machine()
	return self._local_machine
end

function DramaManager:do_sync_state(data, sender)
	self:sync_change_state(data.character, data.state_id, data.tension or 0)
end


function DramaManager:want_to_sync_state(character, state_id, tension)		
	local data = {}
	data.character = character
	data.state_id = state_id
	data.tension = tension		
	self:sync_change_state(character, state_id, tension)
	self._net_action:send_to_peers(data)
end





function DramaManager:on_incharge(enabled)
	self._filtered_state_default = enabled and self._filtered_state_host or self._filtered_state_client
end


function DramaManager:update(t, dt)
	self._t = t
end


function DramaManager:current_tension()
	return self._local_machine:current_tension()
end

function DramaManager:has_local_machine()
	return self._local_machine and true or false
end


function DramaManager:is_runnning()
	return self._local_machine:is_runnning()
end

function DramaManager:get_tension_max()
	return self._local_machine:get_tension_max()
end

function DramaManager:is_hurt()
	return self._local_machine:is_hurt()
end

function DramaManager:get_best_canditate(toggle_name, include_bots, bot_toggle_name)
	local handler = WeightHandler:new()
	for id, cell_user in pairs(managers.cell_user:all_cell_users()) do
		local user_weight = cell_user:slider_value(toggle_name)		
		if user_weight > 0 then
			local user_unit = cell_user:unit()
			if user_unit and alive(user_unit) then
				handler:add_value(user_unit, user_weight)
			end			
		end		
	end	
	
	if Network:is_server() and include_bots then
		local bot_weight = managers.cell_options:slider_value(bot_toggle_name)
		if bot_weight > 0 then
		
			local bots = {}
			for u_key, u_data in pairs(managers.groupai:state():all_AI_criminals()) do
				if u_data.unit and alive(u_data.unit) and not u_data.unit:movement():downed() then
					table.insert(bots, u_data.unit)
				end
			end
			
			if next(bots) then
				local random_index = math.random(1, #bots)
				handler:add_value(bots[random_index], bot_weight)
			end
			
		end
	end
	if handler:has_any() then
		return handler:get_best_value().value
	end
	
	return nil
end

function DramaManager:sync_change_state(character, state_name, tension)	
	self._peer_synced_states[character] = state_name
	self._synced_tensions[character] = tension
	self:debug_print("Syncing character " .. tostring(character) .. " to state name : " ..state_name .. " with tension : " .. tostring(tension))
end

function DramaManager:revive_line_on_downed_teammate(helping_unit, revive_unit)	
	if self._local_machine then
		self._local_machine:revive_line_on_downed_teammate(helping_unit, revive_unit)
	end	
end

function DramaManager:check_synced_states(wanted_state)	
	local synced_states = {}	
	for character, state_name in pairs(self._peer_synced_states) do
		synced_states[state_name] = true
	end	
	if wanted_state then
		return synced_states[wanted_state]	
	end	
	return synced_states
end

function DramaManager:is_character_in_drama_state(character, wanted_state)
	return self._peer_synced_states[character] and self._peer_synced_states[character] == wanted_state or false
end

function DramaManager:states()
	return self._local_machine:states()
end


function DramaManager:force_drama_state(state)	
	if self._local_machine then
		if type(state) == "number"  then
			state = self._local_machine:get_state_name_by_ID(state)
		end
		if self._local_machine:get_state_by_name(state) then
			self._filtered_state_forced:set_forced_state(state)
			self._current_state_filtered = self._filtered_state_forced
			self._has_forced_state = true
			self:change_state(state)
		end
	end

end


function DramaManager:unforce_drama_state()
	self._has_forced_state = false
	self._current_state_filtered = self._filtered_state_default
end


function DramaManager:any_enemies_alarmed()
	for u_key, u_data in pairs(managers.enemy:all_enemies()) do
		if u_data and u_data.unit and alive(u_data.unit) and not u_data.unit:movement():cool() then
			return true
		end
	end	 
	return false
end

function DramaManager:enemies_alarmed_within_range(range)
	local amount = 0
	local player = managers.player:player_unit()
	if player and alive(player) then
		for u_key, u_data in pairs(managers.enemy:all_enemies()) do
			if u_data and u_data.unit and alive(u_data.unit) and not u_data.unit:movement():cool() then
				local distance = mvector3.distance_sq(player:movement():m_head_pos(), u_data.m_pos)
				if distance < range then
					amount = amount + 1				
				end
			end
		end	 
	end	
	return amount
end

function DramaManager:go_to_sneaking()
	return self._current_state_filtered:go_to_sneaking()
end

function DramaManager:go_to_encounter()
	return self._current_state_filtered:go_to_encounter()
end

function DramaManager:go_to_assault()	 
	return self._current_state_filtered:go_to_assault()
end

function DramaManager:go_to_alert()
	return self._current_state_filtered:go_to_alert()
end

function DramaManager:go_to_control()
	return self._current_state_filtered:go_to_control()
end

function DramaManager:go_to_fake_assault()
	return self._current_state_filtered:go_to_fake_assault()
end

function DramaManager:go_to_anticipation()
	return self._current_state_filtered:go_to_anticipation()
end

function DramaManager:is_in_custody()
	return game_state_machine:current_state_name() == "ingame_waiting_for_respawn"
end



function DramaManager:current_local_state()
	return self._local_machine:current_state()
end


function DramaManager:current_local_state_name()
	return self._local_machine:current_state_name()
end


function DramaManager:change_local_state(name, sync)
	if self._local_machine then
		self._local_machine:change_drama_state(name, sync)
	end	
end


function DramaManager:change_character_state(character, state, sync)
	if not self._synced_machines[character] then
		self:debug_print("ERROR : DramaMachine does not exist for character : " .. tostring(character));
		return
	end
	
	
	machine:change_drama_state(state, sync)
end
