PlayerVoiceManager = PlayerVoiceManager or blt_class()
PlayerVoiceManager.DEBUG_ENABLED = false
function PlayerVoiceManager:init()
	blt.xaudio.setup()
	self._voices = {}
	self._interactionExt = nil
	self._voices_by_character = {}
	self._net_action = CellNetValidationAction:new("VOC",callback(self,self,"DoSayLine"))
	self:setup_listener()	
end

function PlayerVoiceManager:debug_print(txt)
	if PlayerVoiceManager.DEBUG_ENABLED then
		log("PlayerVoiceManager - " .. txt)
	end
end
function PlayerVoiceManager:setup_listener()
	self._criminal_added_key = "PlayerVoiceManager_CriminalAdded"
	self._criminal_removed_key = "PlayerVoiceManager_CriminalRemove"
	managers.criminals:add_listener(self._criminal_added_key, {"on_criminal_added"}, callback(self, self, "on_criminal_added"))		
end

function PlayerVoiceManager:get_modified_interaction(interaction_type)
	return self._interactionExt:get_modified_interaction(interaction_type)
end

function PlayerVoiceManager:setup_interation_ext(unit)
	self._interactionExt = PlayerVoiceInteractionExt:new(unit)
end

function PlayerVoiceManager:on_criminal_added(name, unit, peer_id, ai)
	if unit == nil then
		unit = managers.criminals:character_unit_by_name(name)
	end
	
	if unit == nil then
		self:debug_print("ERROR : No unit for the character : " .. tostring(name))
		return
	end
	
	
	local key = unit:key()
	local new_voice = PlayerVoice:new(unit)
	self._voices[key] = new_voice
	self._voices_by_character[name] = new_voice
	
	if unit == managers.player:player_unit() then
		self:setup_interation_ext(unit)	
	end
end


function PlayerVoiceManager:SetCharacterVoiceForDramaState(character, drama_state)
	if self._voices_by_character[character] then
		self:SetVoiceForDramaState(self._voices_by_character[character], drama_state)
	end
end

function PlayerVoiceManager:SetVoiceForDramaState(voice, drama_state)
	local intensitySwitchValue = SD.SWITCHES.INTENSITY_STATE.SWITCH.INTENSITY_1_NEUTRAL
	if drama_state == DramaMachine.STATES.Sneaking then
		intensitySwitchValue = SD.SWITCHES.INTENSITY_STATE.SWITCH.INTENSITY_2_SNEAKING
	elseif drama_state == DramaMachine.STATES.Encounter then
		intensitySwitchValue = SD.SWITCHES.INTENSITY_STATE.SWITCH.INTENSITY_3_ENCOUNTER	
	elseif drama_state == DramaMachine.STATES.FakeAssault or drama_state == DramaMachine.STATES.Assault then
		intensitySwitchValue = SD.SWITCHES.INTENSITY_STATE.SWITCH.INTENSITY_4_COMBAT	
	end
	
	voice:SetIntensitySwitchValue(intensitySwitchValue)
end

function PlayerVoiceManager:WantToSay(character, eventID, startDialog)
	local character_voice = self._voices_by_character[character]
	if character_voice then
		character_voice:SayLine(eventID, startDialog, self._net_action)
	end	
end

function PlayerVoiceManager:DoSayLine(data, sender)
	local character = data.character
	if character and character ~= "nil" then
		local character_voice = self._voices_by_character[character]
		if character_voice then
			character_voice:DoSyncTriggerSound(data)
		end
	end
end
