MUS_Base = MUS_Base or blt_class(MachineState)

function MUS_Base:init()
	self._wrapper = managers.cell_data:wrapper("MusicStateDataBlock")
end

function MUS_Base:post_events(events)
	if events == nil then
		return
	end
	
	for i, eventData in pairs(events) do
		managers.music:post_event(eventData.eventName)
	end
end

function MUS_Base:set_switches(switches)
	if switches == nil then
		return
	end
	
	for i, switchData in pairs(switches) do
		managers.music:set_switch(switchData.switchGroup, switchData.switchName)
	end
end

function MUS_Base:enter(exit_data)
	if self._datablock == nil or not self._datablock.internalEnabled then
		return
	end	
	
	self:set_switches(self._datablock.m_SwitchesOnEnter)
	self:post_events(self._datablock.m_EventsOnEnter)
end

function MUS_Base:exit()
	if self._datablock == nil or not self._datablock.internalEnabled then
		return
	end
	
	self:post_conditional_exit_events(self._datablock.m_EventsOnExit, self._machine:current_stateID())
end


function MUS_Base:state_is_locked()
	if self._datablock == nil then
		return false
	end
	
	if self._datablock.m_timing == nil then
		return false
	end
	
	if self._datablock.m_timing.minTimeInState == nil then
		return false
	end
	
	return self._t - self._state_change_t < self._datablock.m_timing.minTimeInState
end

function MUS_Base:post_conditional_exit_events(events, exitingTo)
	if events == nil then
		return 
	end
	
	if exitingTo == nil then
		return 
	end
	
	
	for i,eventData in pairs(events) do	
		if exitingTo == eventData.onlyWhenExitingTo then
			managers.music:post_event(eventData.eventName);
		end
	end
end

function MUS_Base:update(t,dt)

	if self._datablock == nil or self._datablock.m_timing == nil or self._datablock.m_timing.stateAfterMaxTime == nil then
		return
	end
	
	if self:max_time_reached() and self._datablock.m_timing.stateAfterMaxTime ~= self._machine:get_stateID_by_name(MusicMachine.MUS_States.None) then
		self._machine:change_state(self._datablock.m_timing.stateAfterMaxTime)
		return
	end

	
end

function MUS_Base:max_time_reached()
	if self._datablock == nil then
		return false
	end
	
	if self._datablock.m_timing == nil then
		return false
	end
	
	
	
	if self._datablock.m_timing.maxTimeInState == nil or self._datablock.m_timing.maxTimeInState <= 0 then
		return false
	end
	
	return self._t - self._state_change_t > self._datablock.m_timing.maxTimeInState
end



function MUS_Base:debug_prefix()
	return "MUS_Base"
end

