CellSoundObject = CellSoundObject or blt_class()
CellSoundObject.Currently_Playing = {}

function CellSoundObject.update_soundObjects(t,dt)
	if next(CellSoundObject.Currently_Playing) then
		for i, cellsoundObject in pairs(CellSoundObject.Currently_Playing) do
			if cellsoundObject:Update(t,dt) then
				cellsoundObject:Finished()
			end
		end
	end	
end


function CellSoundObject:init(player_source, unit, sound_path, followUpSoundObjectID, volume, callback)
	self._cell_player_source = player_source
	self._sound_exists = FileUtils:PathExists(sound_path .. ".ogg")
	if not self._sound_exists then
		CellSoundPlayer:DebugPrint("SoundPath does not exist : " ..sound_path .. ".ogg")
	end
	
	self._volume = volume or 0
	self._delay = 0
	self._delayed = false
	self._unit = unit
	self._playing = false
	self._sound_path = CellSoundPlayer.PATH .. sound_path .. ".ogg"
	self._callback = callback
	self._followUpSoundObjectID = followUpSoundObjectID	
	self:Add()
end

function CellSoundObject:set_delay(delay)
	self._delay = delay or 0
	if delay and delay > 0 then
		self._delayed = true
	end
end

function CellSoundObject:SoundExists()
	return self._sound_exists
end


function CellSoundObject:cell_player_source()
	return self._cell_player_source
end

function CellSoundObject:CallBack(reason)
	if self._callback then
		self._callback(reason)
	end
end

function CellSoundObject:Stop()
	self:Delete()
	self._delayed = false
	self._delay = 0	
	if self._sound_source then
		if not self._sound_source:is_closed() then				
			self._sound_source:close()			
		end
	end	
	self:CallBack("stopped")	
end

function CellSoundObject:Add()
	table.insert(CellSoundObject.Currently_Playing, self)
end

function CellSoundObject:Delete()
	table.delete(CellSoundObject.Currently_Playing, self)
end

function CellSoundObject:Finished()
	self:Delete()
	if self._followUpSoundObjectID and self._followUpSoundObjectID > 0 then
		self._cell_player_source:StartTracking()
		if not self._cell_player_source:QueueSoundObject(self._followUpSoundObjectID) then
			self:CallBack("finished")	
		end
	else
		self:CallBack("finished")	
	end	
end

function CellSoundObject:Playing()
	return self._playing
end


function CellSoundObject:Update(t,dt)
	if self._delayed then
		if self._delay > 0 then
			self._delay = self._delay - dt
		else
			self._delayed = false
			self:Play()
		end		
		return false
	end
	return self._sound_source and self._sound_source:is_closed()
end

function CellSoundObject:Play()	
	if not self._sound_exists then
		self:CallBack("finished")
		return
	end
	
	self._playing = true
	
	self._sound_buffer = XAudio.Buffer:new(self._sound_path)
	
	local has_unit = self._unit and self._unit:base()
	local play_2d = not has_unit or self._unit:base().is_local_player
	if play_2d then
		self._sound_source = XAudio.ChatterSource:new(self._sound_buffer)
		self._sound_source:set_relative(true)
	else
		self._sound_source = XAudio.ChatterUnitSource:new(self._unit, self._sound_buffer)
		self._sound_source:set_relative(false)
	end	
	
	
	local volume = self._volume	
	if volume == 0 then
		volume = 2
	end	
	self._sound_source:set_volume(volume, true) 
end