dofile(ModPath .. "Lua/Scripts/Utils/CellSound/CellSoundObject.lua")
CellSoundPlayer = CellSoundPlayer or blt_class()
CellSoundPlayer.DEBUG_ENABLED = true
CellSoundPlayer.PATH = ModPath
CellSoundPlayer.Active_Players = {}
CellSoundPlayer.SOUND_OBJECT_TYPE = {
	Sound = 0,
	Switch = 1,
	Random = 2,
	State = 3,
	RandomChance = 4,
}


function CellSoundPlayer:init(unit)
	self._switches = {}
	self._soundObjects = {}
	self._event_blocks_by_ID = {}
	self._event_blocks_by_Name = {}
	self._switch_blocks = {}	
	self._switchDataWrapper = managers.cell_data:wrapper("SwitchContainerDataBlock")
	self._soundEventDataWrapper = managers.cell_data:wrapper("SoundEventDataBlock")
	self._soundObjectDataWrapper = managers.cell_data:wrapper("SoundObjectDataBlock")
	self._unit = unit	
	if unit then
		self._character_name = managers.criminals:character_name_by_unit(unit)
	end		
	self:setup()
end

function CellSoundPlayer:destroy()
	table.delete(CellSoundPlayer.Active_Players, self)
	self._character_name = nil
end

function CellSoundPlayer:_setup_events()
	local eventBlocks = self._soundEventDataWrapper:get_all_blocks()
	for index, block in pairs(eventBlocks) do
		if self._event_blocks_by_ID[block.persistentID] then
			error("[CellSoundPlayer:setup] block persistentID of " .. block.name .. " has been registered twice!!")
		end
		
		self._event_blocks_by_ID[block.persistentID] = block
		self._event_blocks_by_Name[block.name] = block	
	end
end

function CellSoundPlayer:_setup_switches()
	for _, switchBlock in pairs(self._switchDataWrapper:get_all_blocks()) do
		self._switch_blocks[switchBlock.persistentID] = switchBlock	
	end
end

function CellSoundPlayer:_setup_sound_objects()
	for _, soundObjBlock in pairs(self._soundObjectDataWrapper:get_all_blocks()) do
		local deep_cloned_soundObjectBlock = deep_clone(soundObjBlock)	
		
		for index, Soundobject in pairs(deep_cloned_soundObjectBlock.SoundObjects) do
			Soundobject.Index = index
		end
		
		self._soundObjects[soundObjBlock.persistentID] = deep_cloned_soundObjectBlock
	end
end

function CellSoundPlayer:setup()	
	self:_setup_events()
	self:_setup_switches()
	self:_setup_sound_objects()
	table.insert(CellSoundPlayer.Active_Players, self)
end

function CellSoundPlayer:SetSwitch(switch_groupID, switchID)
	if not switch_groupID  then
		self:DebugPrint("[CellSoundPlayer:SetSwitch] switch_groupID is invalid!!")
		return
	end
	
	
	local switchBlock
	if type(switch_groupID) == "string" then
		switchBlock = self._switchDataWrapper:get_block_by_name(switch_groupID)
		if not switchBlock then
			self:DebugPrint("[CellSoundPlayer:SetSwitch] switchGroup is not valid! : " ..tostring(switch_groupID))
			return
		end		
		switch_groupID = switchBlock.persistentID		
	else
		switchBlock = self._switchDataWrapper:get_block_by_id(switch_groupID)
	end
	
	if not switchBlock then
		self:DebugPrint("[CellSoundPlayer:SetSwitch] switchGroup is not valid! : " ..tostring(switch_groupID))
		return
	end
	
	if not switchID then
		self:DebugPrint("[CellSoundPlayer:SetSwitch] switchID is nil!!")
		return
	end
	
	if type(switchID) == "string" then
		local switchName = switchID
		switchID = nil
		for i,v in pairs(switchBlock.switches) do
			if v.switchName == switchName then
				switchID = i
				break
			end
		end
		
		if not switchID then
			self:DebugPrint("[CellSoundPlayer:SetSwitch] switchName is not valid! : " ..tostring(switchName))
			return
		end
		
	end
	
	
	if not self._switch_blocks[switch_groupID] then
		self:DebugPrint("Incoming invalid switch_groupID " .. tostring(switch_groupID))
		return
	end
	
	self._switches[switch_groupID] = switchID
	
	if switchBlock then
		local switchName = switchBlock:get_switch_name_by_switchID(switchID)		
		self:DebugPrint("<<<<---->>>>\n[Setting Switch]\nSwitchGroupName : " .. tostring(switchBlock.name).."\nSwitchName : "..tostring(switchName))
	end
end

function CellSoundPlayer:StartTracking()
	self._soundObjectIDsTracker = {}
end

function CellSoundPlayer:TrackSoundObjectID(soundObjectId)
	self._soundObjectIDsTracker[soundObjectId] = true
end


function CellSoundPlayer:SoundObjectIDAlreadyTracked(soundObjectId)
	return self._soundObjectIDsTracker[soundObjectId]
end

function CellSoundPlayer:Post(eventID, callback, sync_callback)
	self:StartTracking()
	local eventBlock
	if type(eventID) == "string" then
		eventBlock = self._event_blocks_by_Name[eventID]
	else   
		eventBlock = self._event_blocks_by_ID[eventID]
	end
	if not eventBlock then
		self:DebugPrint("Incoming invalid eventID " .. tostring(eventID))
		return false
	end
	
	self._callback = callback	
	
	self._sync_callback = sync_callback
	
	self._cellObjects = {}
	
	if eventBlock.PlaySoundObjectID and eventBlock.PlaySoundObjectID > 0 then
		self._delay = eventBlock.Delay or 0
		self._volume = eventBlock.Volume or 0
		self:DebugPrint("Triggering  PlaySoundObjectID : " .. tostring(eventBlock.PlaySoundObjectID))
		return self:QueueSoundObject(eventBlock.PlaySoundObjectID)
	end
	
	
	self:DebugPrint("Couldn't play " .. tostring(eventID))
	
	return false	
end

function CellSoundPlayer:DebugPrint(message)
	if CellSoundPlayer.DEBUG_ENABLED then
		log("CellSoundPlayer - " .. message)
	end	
end

function CellSoundPlayer:QueueSoundObject(SoundObjectID)
	local soundObjectBlock = self._soundObjects[SoundObjectID]
	
	if not soundObjectBlock or not soundObjectBlock.internalEnabled then
		self:DebugPrint("soundObjectBlock with the persistentID of " .. SoundObjectID  .. " is invalid!")
		return false
	end
	
	
	if self:SoundObjectIDAlreadyTracked(SoundObjectID) then
		self:DebugPrint("SoundObjectID " .. tostring(SoundObjectID) .. " has been tracked twice!! Block Name " ..tostring(soundObjectBlock.name))
		return false
	end	
	
	self:TrackSoundObjectID(SoundObjectID)	
	
	if soundObjectBlock.SoundObjectType == CellSoundPlayer.SOUND_OBJECT_TYPE.Random then
		return self:OnRandom(soundObjectBlock)
	elseif soundObjectBlock.SoundObjectType == CellSoundPlayer.SOUND_OBJECT_TYPE.Switch then
		return self:OnSwitch(soundObjectBlock)
	elseif soundObjectBlock.SoundObjectType == CellSoundPlayer.SOUND_OBJECT_TYPE.Sound then
		return self:TriggerSound(soundObjectBlock)
	end	
	
	return false
end

function CellSoundPlayer:TriggerSound(soundObjectBlock)	
	local rng = math.random()	
	if soundObjectBlock.RandomChance and soundObjectBlock.RandomChance > 0 and rng > soundObjectBlock.RandomChance then
		self:DebugPrint("[CellSoundPlayer:TriggerSound] random chance failed!!")
		return false
	end
	
	local weightHandler = WeightHandler:new()
	
	local availableSoundObjects = {}
	
	for i, soundObject in pairs(soundObjectBlock.SoundObjects) do		
		if soundObject and soundObject.enabled then		
			table.insert(availableSoundObjects, soundObject)
		end				
	end
	
	for i, soundObject in pairs(availableSoundObjects) do		
		if soundObject and soundObject.enabled then		
			if #availableSoundObjects == 1 or not soundObject.LastPlayed then				
				weightHandler:add_value(soundObject, soundObject.Weight)
			end
			soundObject.LastPlayed = false
		end				
	end
	
	if not weightHandler:has_any() then
		self:DebugPrint("[CellSoundPlayer:TriggerSound] no availableSoundObjects from soundObjectBlock "..tostring(soundObjectBlock.name))
		return false
	end
	
	local bestSoundToPlayObject = weightHandler:get_best_value()	
	if bestSoundToPlayObject and bestSoundToPlayObject.value and bestSoundToPlayObject.value.SoundPath then	
		bestSoundToPlayObject.value.LastPlayed = true
		self:DebugPrint("Now Playing " .. tostring(bestSoundToPlayObject.value.SoundPath))
		
		local volume = 2	
		
		if self._volume and self._volume > 0 then
			volume = self._volume
		elseif soundObjectBlock.Volume and soundObjectBlock.Volume > 0 then
			volume = soundObjectBlock.Volume
		end
		
		local cellsoundObject = CellSoundObject:new(self, self._unit, bestSoundToPlayObject.value.SoundPath, soundObjectBlock.FollowUpSoundObjectID, volume, self._callback)
		if self._delay and self._delay > 0 then
			cellsoundObject:set_delay(self._delay)
		else
			cellsoundObject:Play()		
		end	
		self._delay = nil
		if self._sync_callback and self._character_name then
			self._sync_callback:send_to_peers({			
				character = self._character_name,
				soundObjectID = soundObjectBlock.persistentID,
				play_index = bestSoundToPlayObject.value.Index
			})
		end		
		return true
	end
	
	return false
end

function CellSoundPlayer:DoSyncTriggerSound(data)	
	local soundObjectBlock_id = data.soundObjectBlockID
	local play_index = data.play_index
	if not self._soundObjects[soundObjectBlock_id] and not self._soundObjects[soundObjectBlock_id].SoundObjects then
		self:DebugPrint("[CellSoundPlayer:SyncTriggerSound] invalid soundBlockID to sync incoming "..tostring(soundObjectBlock_id))
		return
	end
	
	if not next(self._soundObjects[soundObjectBlock_id].SoundObjects) then
		self:DebugPrint("[CellSoundPlayer:SyncTriggerSound] no availableSoundObjects from soundObjectBlock "..tostring(self._soundObjects[soundObjectBlock_id].name))
		return
	end
	
	for i, v in pairs(self._soundObjects[soundObjectBlock_id].SoundObjects) do
		self._soundObjects[soundObjectBlock_id].SoundObjects[i].LastPlayed = false
	end
	
	local SyncSoundToPlayObject = self._soundObjects[soundObjectBlock_id].SoundObjects[play_index]
	
	if SyncSoundToPlayObject and SyncSoundToPlayObject.SoundPath then	
		SyncSoundToPlayObject.LastPlayed = true	
		self:DebugPrint("Now Sync Playing " .. tostring(SyncSoundToPlayObject.SoundPath))
		local cellsoundObject = CellSoundObject:new(self._unit, SyncSoundToPlayObject.SoundPath, 0, nil)		
		cellsoundObject:Play()				
	end		
end

function CellSoundPlayer:OnSwitch(soundObjectBlock)	
	if not soundObjectBlock.SwitchContainers or not next(soundObjectBlock.SwitchContainers) then
		self:DebugPrint("FAILED finding suitable switchGroupID for soundObjBlock " .. tostring(soundObjectBlock.name))
		return false
	end
	
	local switchContainerDataBlock = self._switchDataWrapper:get_block_by_id(soundObjectBlock.SwitchGroupID)
	
	if switchContainerDataBlock and self._switches[soundObjectBlock.SwitchGroupID] then
		for i, switchData in pairs(soundObjectBlock.SwitchContainers) do	
			if switchData and (switchData.SwitchID and switchData.SwitchID ~= 0 or switchData.SwitchName and switchData.SwitchName ~= "") then
				if self._switches[soundObjectBlock.SwitchGroupID] == switchData.SwitchID or self._switches[soundObjectBlock.SwitchGroupID] == switchContainerDataBlock:get_switchID_by_switchName(switchData.SwitchName) then
					return self:QueueSoundObject(switchData.SoundObjectID) 
				end
			end			
		end
	end
	
	
	self:DebugPrint("FAILED finding suitable switchGroupID for soundObjBlock " .. tostring(soundObjectBlock.name))
	return false
end

function CellSoundPlayer:Stop()
	if next(CellSoundObject.Currently_Playing) then
		for i, cellsoundObject in pairs(CellSoundObject.Currently_Playing) do
			if cellsoundObject:cell_player_source() == self then
				cellsoundObject:Stop()
			end
		end
	end	
end

function CellSoundPlayer:OnRandom(soundObjectBlock)	
	local weightHandler = WeightHandler:new()
	
	for _, rand in pairs(soundObjectBlock.RandomContainers) do		
		weightHandler:add_value(rand.TriggerSoundObjectID, rand.Weight)
	end
	
	if weightHandler:has_any()  then
		local bestSoundObjectID = weightHandler:get_best_value()
		if bestSoundObjectID then
			return self:QueueSoundObject(bestSoundObjectID.value)
		end			
	end
	
	return false
end