FileUtils = FileUtils or {}
FileUtils.FilesLoaded = {}
FileUtils.PrintAlreadyFiles = false
FileUtils.DEBUG_ENABLED = false
FileUtils.PATH = ModPath

function FileUtils:WriteAllText(filePath, fileContents)
	local out = io.open(FileUtils.PATH .. filePath, "wb")
	out:write(fileContents)
	out:close()
end

function FileUtils:ReadAllText(file_path)
	local path = FileUtils.PATH .. file_path
	if not io.file_is_readable( path ) then
		FileUtils:DebugPrint("[FileUtils:ReadAllText] ERROR : " .. path .. " does not exist!!")
		return		
	end	
	
	local file = io.open( path, "r" )
	if file then
		local file_content = file:read("*all")
		file:close()		
		return file_content
	end
end

function FileUtils:DebugPrint(...)
	if FileUtils.DEBUG_ENABLED then
		log(...)
	end	
end

function FileUtils:LoadFile(file_path, force)
	local path = FileUtils.PATH .. file_path
	
	
	if io.file_is_readable( path ) then		
		if force or not FileUtils.FilesLoaded[path] then
			FileUtils.FilesLoaded[path] = true
			dofile(path)		
		else
			if FileUtils.PrintAlreadyFiles then
				FileUtils:DebugPrint("[FileUtils:LoadFile] FilePath "..path.." was already loaded!")	
			end			
		end
				
		return true
	end	
	
	FileUtils:DebugPrint("[FileUtils:LoadFile] ERROR : " .. path .. " does not exist!!")	
	return false
end

function FileUtils:ExecuteFile(path, commands)
	os.execute("\""..FileUtils.PATH .. path.."\" "..commands);
end

function FileUtils:ReadAllBytes(file_path)
	return ByteTools:GetBytes(self:ReadAllText(file_path))
end

function FileUtils:DirectoryExists(file_path)
	return file.DirectoryExists(FileUtils.PATH .. file_path)
end

function FileUtils:PathExists(file_path)
	return io.file_is_readable(FileUtils.PATH .. file_path)
end

function FileUtils:save_hash_file_text()
	local hash = tostring(file.DirectoryHash(FileUtils.PATH))
	local file = io.open("DeadLocke_hash.txt", "w+")
	file:write(hash)
	file:close()
end

function FileUtils:sounds_directory(...)
	local params = {...}
	local amount = select("#",...)
	
	local path = self._path.."assets/sounds/"
	
	for i,v in pairs(params) do
		path = path..v..(amount == i and "" or "/")		
	end
	
	return path..".ogg"
end

function FileUtils:CreateDirectory(file_path)
	local path = FileUtils.PATH .. file_path
	os.execute( path )
end

function FileUtils:CombinePath(...)
	local params = {...}
	local amount = select("#",...)
	
	local path = FileUtils.PATH
	
	for i,v in pairs(params) do
		path = path..v..(amount == i and "" or "/")		
	end	
	
	
	local possible_directory = path.."/"
	if file.DirectoryExists(possible_directory) then
		return possible_directory
	end
	
	return path
end