StateMachine = StateMachine or blt_class()
StateMachine.active_machines = {}

function StateMachine:init(...)
	self._state_data = {}
	self._current_state = nil
	self._last_state = nil
	self._states = {}	
	self._state_by_ID = {}	
	self._state_ID_by_name = {}	
	self._state_name_by_ID = {}	
	self._start_state = nil
	self._debug_enabled = false
	self._t = 0
	self._last_state_ID = -1
	self:register()
end

function StateMachine._register_machine(machine)
	table.insert(StateMachine.active_machines, machine)
end

function StateMachine._unregister_machine(machine)	
	table.delete(StateMachine.active_machines, machine)
end

function StateMachine:register()
	self._active = true
	StateMachine._register_machine(self)
end

function StateMachine:unregister()
	self._active = false
	StateMachine._unregister_machine(self)
end

function StateMachine:active()
	return self._active
end

function StateMachine.update_machines(t,dt)
	for i, machine in pairs(StateMachine.active_machines) do
		if machine then
			machine:update_time(t,dt)
			machine:update(t,dt)
		end		
	end
end


function StateMachine:is_locally_owned()
	return true
end

function StateMachine:is_debug_enabledd()
	return self._debug_enabledd
end

function StateMachine:update(t,dt)
	self:update_state(t,dt)
end

function StateMachine:update_time(t)
	self._t = t
end

function StateMachine:override_state(from, to)
	state_instance = self._states[from]
	
	self._states[to] = state
end


function StateMachine:add_state(name, state_instance)
	self._last_state_ID = self._last_state_ID + 1
	
	local stateID = self._last_state_ID
	
	if self._debug_enabled then
		--log("adding state " .. name)
	end
	
	state_instance.DEBUG_ENABLED = self._debug_enabled
	state_instance:set_machine(self)	
	state_instance.STATE_NAME = name
	state_instance.STATE_ID = stateID	
	self:set_state(name, stateID, state_instance)
	state_instance:setup()
	return state_instance
end

function StateMachine:set_state(name, stateID, state_instance)
	self._states[name] = state_instance
	self._state_by_ID[stateID] = state_instance
	self._state_ID_by_name[name] = stateID
	self._state_name_by_ID[stateID] = name
end


function StateMachine:enable_debug(enable)
	self._debug_enabled = enable
	for name, state in pairs(self._states) do
		if state then
			state:enable_debug(enable)
		end		
	end
end

function StateMachine:state_enter_t()
	return self._state_enter_t
end

function StateMachine:current_state()
	return self._current_state
end

function StateMachine:current_state_name()
	return self._current_state and self._current_state.STATE_NAME or nil
end

function StateMachine:current_stateID()
	return self._current_state and self._current_state.STATE_ID or nil
end

function StateMachine:last_state()
	return self._last_state
end

function StateMachine:last_state_name()
	return self._last_state and self._last_state.STATE_NAME or nil
end

function StateMachine:last_stateID()
	return self._last_state and self._last_state.STATE_ID or nil
end

function StateMachine:reset()
	self:_change_state(self._start_state)
end

function StateMachine:set_start_state(state)
	self._start_state = state
	self:_change_state(self._start_state)
end

function StateMachine:states()
	return self._states
end

function StateMachine:get_state_by_name(name)
	return self._states[name]
end

function StateMachine:get_state_by_ID(stateID)
	return self._state_by_ID[stateID]
end

function StateMachine:get_stateID_by_name(name)
	return self._state_ID_by_name[name]
end

function StateMachine:get_state_name_by_ID(stateID)
	return self._state_name_by_ID[stateID]
end


function StateMachine:_change_state(new_state)
	local exit_data = nil	
	
	local is_locally_owned = self:is_locally_owned()
	
	self._last_state = self._current_state	
	self._state_enter_t = self._t
	self._current_state = new_state
	self._current_state._state_change_t = self._state_enter_t
	
	if self._last_state then	
		if is_locally_owned then
			exit_data = self._last_state:exit()
		else
			exit_data = self._last_state:sync_exit()
		end	
	end	
	
	
	if is_locally_owned then		
		new_state:enter(exit_data)
	else
		new_state:sync_enter(exit_data)
	end		
	
end

function StateMachine:change_state(id)
	local new_state
	
	if id then
		if type(id) == "string" then
			new_state = self:get_state_by_name(id)	
		elseif type(id) == "number" then
			new_state = self:get_state_by_ID(id)	
		end
	end	
	
	if not new_state then
		log("[StateMachine:change_state] state : " .. tostring(id).. " is invalid!")
		return
	end
	
	self:_change_state(new_state)
end

function StateMachine:change_state_by_ID(id)
	self:change_state(id)
end


function StateMachine:update_state(t,dt)
	if not self._current_state then
		return
	end
	
	self._current_state:update_time(t)
	
	if self:is_locally_owned() then
		self._current_state:update(t,dt)
		return
	end
	
	self._current_state:sync_update(t,dt)
end


